#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "parser.h"
#include <QErrorMessage>
#include <QPushButton>
#include <QFont>
#include <QtXmlPatterns>
#include <QContextMenuEvent>

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
    , $file(nullptr)
    , $pos(nullptr)
{
    ui->setupUi(this);
    treeWidget = new QTreeWidget(this);
    treeWidget->setHeaderLabel("Файлы");
    treeWidget->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(treeWidget, SIGNAL(customContextMenuRequested(const QPoint&)),
            this, SLOT(Custom_menu(const QPoint&)));
    menuCustom = new QMenu(this);
    menuCustom->addAction("&Обратиться", this, SLOT(Refer_to()));
    menuCustom->addAction("&Закрыть", this, SLOT(Close_the_file()));
    textEdit = new QTextEdit(this);
    QMenu *menu = new QMenu("&Меню", this);
    QPushButton *putton_download = new QPushButton("Загрузить запрос из файла",this);
    putton_download->setFont(*(new QFont("Times New Roman", 14, QFont::Cursive)));
    connect(putton_download, SIGNAL(clicked()), SLOT(Open_Query()));
    QPushButton *putton = new QPushButton("Обработать",this);
    putton->setFont(*(new QFont("Times New Roman", 14, QFont::Cursive)));
    connect(putton, SIGNAL(clicked()), SLOT(Processing_Query()));
    menu->addAction("&Открыть файл", this, SLOT(Open_the_file()), Qt::CTRL + Qt::Key_O);
    menu->addAction("&Закрыть все файлы", this, SLOT(Clearing()));
    menu->addAction("&Выполнить XQuery Запрос", this, SLOT(Open_Query()), Qt::CTRL + Qt::Key_E);
    menu->addSeparator();
    menu->addAction("&Выход", this, SLOT(Close_the_window()), Qt::CTRL + Qt::Key_Q);
    ui->menubar->addMenu(menu);

    dock = new QDockWidget("Результат выполнения", this);
    textEditForDock = new QTextEdit(this);
    textEditForDock->setReadOnly(true);
    dock->setWidget(textEditForDock);
    mainLayout = new QGridLayout(this);
    mainLayout->addWidget(treeWidget, 0, 0, 6, 3);
    mainLayout->addWidget(textEdit, 6, 0, 2, 3);
    mainLayout->addWidget(putton, 8, 1, 1, 1);
    mainLayout->addWidget(putton_download, 9, 1, 1, 1);
    mainLayout->addWidget(dock,8,0,2,1);
    mainWidget = new QWidget(this);
    mainWidget->setLayout(mainLayout);
    setCentralWidget(mainWidget);
    setWindowTitle("File XML reader pro");
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::Close_the_window()
{
    QApplication::quit();
}

void MainWindow::Clearing()
{
    if($file)
    {
        delete $file;
        $file = nullptr;
        $pos = nullptr;
    }
    treeWidget->clear();
}

void MainWindow::Processing_Query()
{  
    if($file->open(QIODevice::ReadOnly))
    {
        QString query = textEdit->toPlainText();
        QXmlQuery xml_query(QXmlQuery::XQuery10);
        if(!xml_query.setFocus($file))
             (new QErrorMessage(this))->showMessage("Выберите таблицу данных!");
        xml_query.setQuery(query);

        if (!xml_query.isValid())
        {
            $file->close();
            (new QErrorMessage(this))->showMessage("Запрос не корректен!");
        }
        else
        {
            QString answer;
            if (!xml_query.evaluateTo(&answer))
            {
                qDebug() << answer;
                $file->close();
                (new QErrorMessage(this))->showMessage("Ошибка выполнения запроса...");
            }
            else
            {
                $file->close();
                textEditForDock->setText(answer);
                dock->show();
            }
        }
    }
}

void MainWindow::Open_the_file()
{
    QString filepath = QFileDialog::getOpenFileName(this, "Choose file", "File", "*.xml");
    if(!filepath.isEmpty())
    {
        QFile file(filepath);
        QXmlSimpleReader read;
        QString filename = filepath.section("/",-1,-1);
        parser handler(treeWidget, filename);
        QXmlInputSource source(&file);
        read.setContentHandler(&handler);
        if (!read.parse(source))
            (new QErrorMessage(this))->showMessage("Не получается открыть файл!");
        if(!$file)
        {
            $file = new QFile(filepath);
            $pos = treeWidget->itemAt(0, 0);
            $pos->setFont(0, QFont("Times New Roman", 14, QFont::Cursive));
        }
    }
}

void MainWindow::Open_Query()
{
    QString filepath = QFileDialog::getOpenFileName(this, "Choose query-file", "File", "*.xq");
    if (!filepath.isEmpty())
    {
        QFile file(filepath);
        if (file.open(QIODevice::ReadOnly))
        {
            QString query = file.readAll();
            file.close();
            textEdit->setText(query);
            Processing_Query();
        }
        else
        {
            (new QErrorMessage(this))->showMessage("Файл не открывается");
        }
    }
}

void MainWindow::Custom_menu(const QPoint& pos)
{
    if(treeWidget->currentItem() && !treeWidget->currentItem()->parent())
        menuCustom->exec(QCursor::pos());
}

void MainWindow::Refer_to()
{
    QString _currentFileName =  $file->fileName();
    _currentFileName.resize($file->fileName().lastIndexOf('/') + 1);
    _currentFileName += treeWidget->currentItem()->data(0,0).toString();
    $file->setFileName(_currentFileName);
    $pos->setFont(0, QFont("Times New Roman", 12));
    treeWidget->currentItem()->setFont(0, QFont("Times New Roman", 12, QFont::Bold));
    $pos = treeWidget->currentItem();
}

void MainWindow::Close_the_file()
{
    QString newFileName =  $file->fileName();
    newFileName.resize($file->fileName().lastIndexOf('/') + 1);

    QModelIndex _point;
    if(treeWidget->currentIndex().row())
        _point = treeWidget->model()->index(treeWidget->currentIndex().row() - 1,
                                                          treeWidget->currentIndex().column());

    else
        _point = treeWidget->model()->index(treeWidget->currentIndex().row() + 1,
                                                          treeWidget->currentIndex().column());

    QString currnetActiveFile(treeWidget->currentItem()->text(0));
    delete treeWidget->currentItem();

    if(!_point.isValid())
    {
        delete $file;
        $file = nullptr;
    }
    else
    {
        treeWidget->setCurrentIndex(_point);
        newFileName += treeWidget->currentItem()->data(0,0).toString();
        qDebug() << newFileName;
        if($file->fileName().contains(currnetActiveFile))
        {
            $file->setFileName(newFileName);
            treeWidget->currentItem()->setFont(0, QFont("Times New Roman", 14, QFont::Cursive));
        }
        $pos = treeWidget->currentItem();
    }
}
